package org.eclipse.jdt.internal.compiler.classfmt;

package .tmp;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.classfmt.ExternalAnnotationProvider;
import org.eclipse.jdt.internal.compiler.env.IBinaryAnnotation;
import org.eclipse.jdt.internal.compiler.env.ITypeAnnotationWalker;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.SignatureWrapper;

abstract class BasicAnnotationWalker implements ITypeAnnotationWalker {
  char[] source;
  
  SignatureWrapper wrapper;
  
  int pos;
  
  int prevTypeArgStart;
  
  int currentTypeBound;
  
  LookupEnvironment environment;
  
  BasicAnnotationWalker(char[] source, int pos, LookupEnvironment environment) {
    this.source = source;
    this.pos = pos;
    this.environment = environment;
    paramExternalAnnotationProvider.initAnnotations(environment);
  }
  
  SignatureWrapper wrapperWithStart(int start) {
    if (this.wrapper == null)
      this.wrapper = new SignatureWrapper(this.source); 
    this.wrapper.start = start;
    return this.wrapper;
  }
  
  public ITypeAnnotationWalker toReceiver() {
    return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
  }
  
  public ITypeAnnotationWalker toTypeParameter(boolean isClassTypeParameter, int rank) {
    return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
  }
  
  public ITypeAnnotationWalker toTypeParameterBounds(boolean isClassTypeParameter, int parameterRank) {
    return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
  }
  
  public ITypeAnnotationWalker toTypeBound(short boundIndex) {
    return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
  }
  
  public ITypeAnnotationWalker toSupertype(short index, char[] superTypeSignature) {
    return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
  }
  
  public ITypeAnnotationWalker toTypeArgument(int rank) {
    if (rank == 0) {
      int start = CharOperation.indexOf('<', this.source, this.pos) + 1;
      this.prevTypeArgStart = start;
      return (ITypeAnnotationWalker)new ExternalAnnotationProvider.MethodAnnotationWalker(ExternalAnnotationProvider.this, this.source, start, this.environment);
    } 
    int next = this.prevTypeArgStart;
    switch (this.source[next]) {
      case '*':
        this.prevTypeArgStart = ++next;
        return (ITypeAnnotationWalker)new ExternalAnnotationProvider.MethodAnnotationWalker(ExternalAnnotationProvider.this, this.source, next, this.environment);
      case '+':
      case '-':
        next++;
        break;
    } 
    next = wrapperWithStart(next).computeEnd();
  }
  
  public ITypeAnnotationWalker toWildcardBound() {
    switch (this.source[this.pos]) {
      case '+':
      case '-':
        return (ITypeAnnotationWalker)new ExternalAnnotationProvider.MethodAnnotationWalker(ExternalAnnotationProvider.this, this.source, this.pos + 1, this.environment);
    } 
    return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
  }
  
  public ITypeAnnotationWalker toNextArrayDimension() {
    if (this.source[this.pos] == '[') {
      int newPos = this.pos + 1;
      switch (this.source[newPos]) {
        case '0':
        case '1':
          newPos++;
          break;
      } 
      return (ITypeAnnotationWalker)new ExternalAnnotationProvider.MethodAnnotationWalker(ExternalAnnotationProvider.this, this.source, newPos, this.environment);
    } 
    return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
  }
  
  public ITypeAnnotationWalker toNextNestedType() {
    return this;
  }
  
  public IBinaryAnnotation[] getAnnotationsAtCursor(int currentTypeId) {
    if (this.pos != -1 && this.pos < this.source.length - 2)
      switch (this.source[this.pos]) {
        case '*':
        case '+':
        case '-':
        case 'L':
        case 'T':
        case '[':
          switch (this.source[this.pos + 1]) {
            case '0':
              return new IBinaryAnnotation[] { (IBinaryAnnotation)this.this$0.NULLABLE_ANNOTATION };
            case '1':
              return new IBinaryAnnotation[] { (IBinaryAnnotation)this.this$0.NONNULL_ANNOTATION };
          } 
          break;
      }  
    return NO_ANNOTATIONS;
  }
}
